from module.utils import * 
import click,os



def DES3_encrypt(mpath:str):
    if os.path.exists(mpath):
        key1,key2=generateKey()
        click.echo("Save your keys to use them in decryption... \nkey1= "+str(key1)+"\n key2= "+str(key2))
        with open(mpath, 'rb') as input_file:
            fileByte = input_file.read()
            output=desEncrypt(fileByte,key1)
            output= desDecncrypt(output,key2)
            output= desEncrypt(output,key1)
            with open(mpath, 'wb') as output_file:
                output_file.write(output)
                output_file.close()
        click.echo("File "+mpath.split("\\")[-1]+" encrypted successfully")


def DES3_encrypt2k(mpath:str,key1:bytes,key2:bytes):
    if checkKey(key1) and checkKey(key2):
        with open(mpath, 'rb') as input_file:
                fileByte = input_file.read()
                output=desEncrypt(fileByte,key1)
                output= desDecncrypt(output,key2)
                output= desEncrypt(output,key1)
                with open(mpath, 'wb') as output_file:
                    output_file.write(output)
                    output_file.close()
        click.echo("File "+mpath.split("\\")[-1]+" encrypted successfully")
    else: click.echo("Error! key must be at least 8 character")


def DES3_encrypt3k(mpath:str,key1:bytes,key2:bytes,key3:bytes):
    if checkKey(key1) and checkKey(key2) and checkKey(key3):
        with open(mpath, 'rb') as input_file:
                fileByte = input_file.read()
                output=desEncrypt(fileByte,key1)
                output= desDecncrypt(output,key2)
                output= desEncrypt(output,key3)
                with open(mpath, 'wb') as output_file:
                    output_file.write(output)
                    output_file.close()
        click.echo("File "+mpath.split("\\")[-1]+" encrypted successfully")
    else: click.echo("Error! key must be at least 8 character")


def DES3_decrypt2k(mpath:str,key1:bytes,key2:bytes):
    if checkKey(key1) and checkKey(key2):
        with open(mpath, 'rb') as input_file:
                fileByte = input_file.read()
                output=desDecncrypt(fileByte,key1)
                output= desEncrypt(output,key2)
                output= desDecncrypt(output,key1)
                with open(mpath, 'wb') as output_file:
                    output_file.write(output)
                    output_file.close()
        click.echo("File "+mpath.split("\\")[-1]+" decrypted successfully")
    else: click.echo("Error! key must be at least 8 character")


def DES3_decrypt3k(mpath:str,key1:bytes,key2:bytes,key3:bytes):
    if checkKey(key1) and checkKey(key2) and checkKey(key3):
        with open(mpath, 'rb') as input_file:
                fileByte = input_file.read()
                output=desDecncrypt(fileByte,key3)
                output= desEncrypt(output,key2)
                output= desDecncrypt(output,key1)
                with open(mpath, 'wb') as output_file:
                    output_file.write(output)
                    output_file.close()
        click.echo("File "+mpath.split("\\")[-1]+" decrypted successfully")
    else: click.echo("Error! key must be at least 8 character")


def DES3_encrypt_all(mpath:str):
    if os.path.exists(mpath):
        key1,key2=generateKey()
        click.echo("Save your keys to use them in decryption... \nkey1= "+str(key1)+"\n key2= "+str(key2))
        for dirpath, dirnames, filenames in os.walk(mpath):
            for file in filenames:
                DES3_encrypt2k(os.path.join(dirpath,file),key1=key1,key2=key2)
    else: click.echo("Error! Directory Not Found")       



def DES3_encrypt_all2k(mpath:str,key1:bytes,key2:bytes):
    if checkKey(key1) and checkKey(key2):
        if os.path.exists(mpath):
            for dirpath, dirnames, filenames in os.walk(mpath):
                for file in filenames:
                    DES3_encrypt2k(os.path.join(dirpath,file),key1=key1,key2=key2)
        else: click.echo("Error! Directory Not Found")   
    else: click.echo("Error! key must be at least 8 character")
 


def DES3_encrypt_all3k(mpath:str,key1:bytes,key2:bytes,key3:bytes):
    if checkKey(key1) and checkKey(key2) and checkKey(key3):
        if os.path.exists(mpath):
            for dirpath, dirnames, filenames in os.walk(mpath):
                for file in filenames:
                    DES3_encrypt3k(os.path.join(dirpath,file),key1=key1,key2=key2,key3=key3)
        else: click.echo("Error! Directory Not Found")   
    else: click.echo("Error! key must be at least 8 character")




def DES3_decrypt_all2k(mpath:str,key1:bytes,key2:bytes):
    if checkKey(key1) and checkKey(key2):
        if os.path.exists(mpath):
            for dirpath, dirnames, filenames in os.walk(mpath):
                for file in filenames:
                    DES3_decrypt2k(os.path.join(dirpath,file),key1=key1,key2=key2)
        else: click.echo("Error! Directory Not Found")
    else: click.echo("Error! key must be at least 8 character")



def DES3_decrypt_all3K(mpath:str,key1:bytes,key2:bytes,key3:bytes):
    if checkKey(key1) and checkKey(key2) and checkKey(key3):
        if os.path.exists(mpath):
            for dirpath, dirnames, filenames in os.walk(mpath):
                for file in filenames:
                    DES3_decrypt3k(os.path.join(dirpath,file),key1=key1,key2=key2,key3=key3)
        else: click.echo("Error! Directory Not Found")
    else: click.echo("Error! key must be at least 8 character")



import click
from module import DES3 as des3



@click.group() 
def cli():
    '''
    tdes is a tool for encryption and decryption files that use triple Data Encryption Standard aka 3DES
    '''
    pass



@cli.command()
@click.option('--path',  help='path of the file you want to encrypt',required=True)
@click.option('--key1',  help='The First key to use in 3DES encryption make sure to save it for decryption and enter it in the same order',required=False)
@click.option('--key2',  help='The second key to use in 3DES encryption make sure to save it for decryption and enter it in the same order',required=False)
@click.option('--key3',  help='The third key to use in 3DES encryption make sure to save it for decryption and enter it in the same order.',required=False)
def encrypt(path,key1=None,key2=None,key3=None):
    '''
    encrypt one file by given its path [required] and keys for encryption [optional]
    '''
    if key1 and key2 and key3:
        des3.DES3_encrypt3k(path,key1.encode(),key2.encode(), key3.encode())
    elif key1 and key2:
        des3.DES3_encrypt2k(path,key1.encode(),key2.encode())
    else:
        des3.DES3_encrypt(path)


@cli.command()
@click.option('--path',  help='path of the file you want to decrypt',required=True)
@click.option('--key1',  help='The First key to use in 3DES decryption make sure to enter it in the same order of encryption',required=True)
@click.option('--key2',  help='The second key to use in 3DES decryption make sure to enter it in the same order of encryption.',required=True)
@click.option('--key3',  help='The third key to use in 3DES decryption make sure to enter it in the same order of encryption.',required=False)
def decrypt(path,key1,key2,key3=None):
    '''
    decrypt one file by given its path [required] and keys for encryption [optional]
    '''
    if key3:
        des3.DES3_decrypt3k(path,key1=key1.encode(),key2=key2.encode(),key3=key3.encode())
    else:
        des3.DES3_decrypt2k(path,key1=key1.encode(),key2=key2.encode())
  

@cli.command()
@click.option('--path',  help='path of the directory you want to encrypt it\'s files',required=True) 
@click.option('--key1',  help='The First key to use in 3DES encryption make sure to save it for decryption and enter it in the same order',required=False)
@click.option('--key2',  help='The second key to use in 3DES encryption make sure to save it for decryption and enter it in the same order',required=False)
@click.option('--key3',  help='The third key to use in 3DES encryption make sure to save it for decryption and enter it in the same order.',required=False)
def encryptall(path,key1=None,key2=None,key3=None):
    '''
    encrypt all files in a directory by given its path [required] and keys for encryption [optional]
    '''
    if key1 and key2 and key3:
        des3.DES3_encrypt_all3k(path,key1.encode(),key2.encode(), key3.encode())
    elif key1 and key2:
        des3.DES3_encrypt_all2k(path,key1.encode(),key2.encode())
    else:
        des3.DES3_encrypt_all(path)


@cli.command()
@click.option('--path',  help='path of the directory you want to encrypt it\'s files',required=True) 
@click.option('--key1',  help='The First key to use in 3DES encryption make sure to save it for decryption and enter it in the same order',required=True)
@click.option('--key2',  help='The second key to use in 3DES encryption make sure to save it for decryption and enter it in the same order',required=True)
@click.option('--key3',  help='The third key to use in 3DES encryption make sure to save it for decryption and enter it in the same order.',required=False)
def decryptall(path,key1,key2,key3=None):
    '''
    decrypt all files in a directory by given its path [required] and keys for encryption [optional]
    '''
    if key3:
        des3.DES3_decrypt_all3K(path,key1.encode(),key2.encode(), key3.encode())
    else :
        des3.DES3_decrypt_all2k(path,key1.encode(),key2.encode())


if __name__=='__main__':
    cli()




from setuptools import setup

with open("README.md","r") as fh:
    long_description=fh.read()

setup(name="tdes",version='0.0.2',
description="encrypting and decrypting files tool using triple DES",
package_dir={'': 'src'}, py_modules=["main","module.DES3","module.utils"],
long_description=long_description,
long_description_content_type="text/markdown",
install_requires=[
    'click',
    'pycryptodomex'],
    entry_points={
        'console_scripts': [
            'tdes = main:cli',
        ]
    },
    classifiers=[
        "Topic :: Security :: Cryptography",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8"
    ],
    url="https://github.com/4bd4ll4h/tdes",
    author="4bd4ll4h",
    author_email="4bd4ll4h.m@gmail.com"
    )


import random,string
from Cryptodome.Cipher import DES


def desEncrypt(file:bytes, key: bytes):
    cipher = DES.new(key=key, mode=DES.MODE_CFB,iv=key)
    return cipher.encrypt(file)


def desDecncrypt(file: str, key: bytes):
     cipher = DES.new(key=key, mode=DES.MODE_CFB,iv=key)
     return cipher.decrypt(file)



def generateKey():
    key1="".join(random.choices(string.ascii_letters+string.digits,k=8))
    key2="".join(random.choices(string.ascii_letters+string.digits,k=8))
    return key1.encode(),key2.encode()



def checkKey(key:bytes):
    return len(key)>=8
